@@ -23,6 +23,8 @@
 #include <KItinerary/IataBcbpParser>
 #include <KItinerary/PdfDocument>
 #include <KItinerary/Uic9183Parser>
+#include <KItinerary/VdvTicket>
+#include <KItinerary/VdvTicketParser>
 
 #include <QImage>
 
@@ -102,6 +104,9 @@ QVariant JsApi::Barcode::decodeUic9183(const QVariant &s) const
     Uic9183Parser p;
     p.setContextDate(m_contextDate);
     p.parse(s.toByteArray());
+    if (!p.isValid()) {
+        return {};
+    }
     return QVariant::fromValue(p);
 }
 
@@ -110,11 +115,25 @@ QVariant JsApi::Barcode::decodeIataBcbp(const QString &s) const
     return QVariant::fromValue(IataBcbpParser::parse(s, m_contextDate.date()));
 }
 
+QVariant JsApi::Barcode::decodeVdvTicket(const QVariant &s) const
+{
+    VdvTicketParser p;
+    if (!p.parse(s.toByteArray())) {
+        return {};
+    }
+    return QVariant::fromValue(p.ticket());
+}
+
 QString JsApi::Barcode::toBase64(const QVariant &b) const
 {
     return QString::fromUtf8(b.toByteArray().toBase64());
 }
 
+QVariant JsApi::Barcode::fromBase64(const QString &s) const
+{
+    return QByteArray::fromBase64(s.toUtf8());
+}
+
 QVariant JsApi::Barcode::toBitArray(const QVariant &b) const
 {
     return QVariant::fromValue(JsApi::BitArray(b.toByteArray()));
