@@ -16,6 +16,7 @@
 */
 
 #include "jsonldimportfilter.h"
+#include "logging.h"
 
 #include <QDebug>
 #include <QJsonArray>
@@ -33,27 +34,37 @@ static const struct {
     const char* fromType;
     const char* toType;
 } type_mapping[] = {
+    { "AutoDealer", "LocalBusiness" },
+    { "AutoRepair", "LocalBusiness" },
+    { "AutomotiveBusiness", "LocalBusiness" },
     { "Bakery", "FoodEstablishment" },
     { "BarOrPub", "FoodEstablishment" },
     { "BedAndBreakfast", "LodgingBusiness" },
     { "Brewery", "FoodEstablishment" },
     { "BusStop", "BusStation" },
+    { "BusinessEvent", "Event" },
     { "CafeOrCoffeeShop", "FoodEstablishment" },
     { "Campground", "LodgingBusiness" },
     { "ChildrensEvent", "Event" },
     { "ComedyEvent", "Event" },
+    { "ComputerStore", "LocalBusiness" },
     { "DanceEvent", "Event" },
     { "Distillery", "FoodEstablishment" },
     { "EditAction", "UpdateAction" },
     { "EducationEvent", "Event" },
+    { "ElectronicsStore", "LocalBusiness" },
+    { "EntertainmentBusiness", "LocalBusiness" },
     { "ExhibitionEvent", "Event" },
     { "FastFoodRestaurant", "FoodEstablishment" },
     { "Festival", "Event" },
+    { "HobbyShop", "LocalBusiness" },
+    { "HomeAndConstructionBusiness", "LocalBusiness" },
     { "Hostel", "LodgingBusiness" },
     { "Hotel", "LodgingBusiness" },
     { "IceCreamShop", "FoodEstablishment" },
     { "LiteraryEvent", "Event" },
     { "Motel", "LodgingBusiness" },
+    { "MovieTheater", "LocalBusiness" },
     { "MusicEvent", "Event" },
     { "Resort", "LodgingBusiness" },
     { "Restaurant", "FoodEstablishment" },
@@ -61,6 +72,7 @@ static const struct {
     { "ScreeningEvent", "Event" },
     { "SocialEvent", "Event" },
     { "SportsEvent", "Event" },
+    { "Store", "LocalBusiness" },
     { "TheaterEvent", "Event" },
     { "VisualArtsEvent", "Event" },
     { "Winery", "FoodEstablishment" },
@@ -137,6 +149,24 @@ static void filterReservation(QJsonObject &res)
         }
     }
 
+    // unpack reservationFor array - if we ever encounter more than one element in here we'd need to multiply the result
+    const auto resFor = res.value(QLatin1String("reservationFor"));
+    if (resFor.isArray()) {
+        const auto a = resFor.toArray();
+        if (a.size() > 1) {
+            qCWarning(Log) << "Found reservationFor array with" << a.size() << "elements!";
+        }
+        if (!a.isEmpty()) {
+            res.insert(QStringLiteral("reservationFor"), a.at(0));
+        }
+    }
+
+    // normalize reservationStatus enum
+    auto resStat = res.value(QLatin1String("reservationStatus")).toString();
+    if (!resStat.isEmpty() && !resStat.contains(QLatin1String("/Reservation"))) {
+        res.insert(QStringLiteral("reservationStatus"), resStat.replace(QLatin1String("http://schema.org/"), QLatin1String("http://schema.org/Reservation")));
+    }
+
     // legacy potentialAction property
     renameProperty(res, "action", "potentialAction");
 
@@ -161,6 +191,22 @@ static void filterBusTrip(QJsonObject &trip)
     renameProperty(trip, "busCompany", "provider");
 }
 
+static void filterFoodEstablishment(QJsonObject &restaurant)
+{
+    // This can be a bool, "Yes"/"No", or a URL.
+    auto reservationsValue = restaurant.value(QLatin1String("acceptsReservations"));
+    if (reservationsValue.isString()) {
+        const QString reservations = reservationsValue.toString();
+        if (reservations == QLatin1String("Yes")) {
+            restaurant.insert(QLatin1String("acceptsReservations"), true);
+        } else if (reservations == QLatin1String("No")) {
+            restaurant.insert(QLatin1String("acceptsReservations"), false);
+        } else {
+            migrateToAction(restaurant, "acceptsReservations", "ReserveAction", true);
+        }
+    }
+}
+
 static void filterActionTarget(QJsonObject &action)
 {
     QJsonArray targets;
@@ -241,6 +287,7 @@ static const struct {
 } type_filters[] = {
     { "BusTrip", filterBusTrip },
     { "Flight", filterFlight },
+    { "FoodEstablishment", filterFoodEstablishment },
     { "LodgingReservation", filterLodgingReservation },
     { "TrainTrip", filterTrainTrip },
 };
@@ -264,14 +311,15 @@ static void filterRecursive(QJsonArray &array)
 
 static void filterRecursive(QJsonObject &obj)
 {
-    const auto type = obj.value(QLatin1String("@type")).toString().toUtf8();
+    auto type = obj.value(QLatin1String("@type")).toString().toUtf8();
 
     // normalize type
     const auto it = std::lower_bound(std::begin(type_mapping), std::end(type_mapping), type, [](const auto &lhs, const auto &rhs) {
         return std::strcmp(lhs.fromType, rhs.constData()) < 0;
     });
     if (it != std::end(type_mapping) && std::strcmp((*it).fromType, type.constData()) == 0) {
-        obj.insert(QStringLiteral("@type"), QLatin1String((*it).toType));
+        type = it->toType;
+        obj.insert(QStringLiteral("@type"), QLatin1String(type));
     }
 
     for (auto it = obj.begin(); it != obj.end(); ++it) {
@@ -295,20 +343,70 @@ static void filterRecursive(QJsonObject &obj)
     }
 }
 
-QJsonObject JsonLdImportFilter::filterObject(const QJsonObject& obj)
+static QJsonArray graphExpand(const QJsonObject &obj)
 {
-    QJsonObject res(obj);
-    filterRecursive(res);
+    QJsonArray result;
 
-    const auto type = obj.value(QLatin1String("@type")).toString();
-    if (type.endsWith(QLatin1String("Reservation"))) {
-        filterReservation(res);
+    const auto graph = obj.value(QLatin1String("@graph")).toArray();
+    for (const auto &o : graph) {
+        const auto a = JsonLdImportFilter::filterObject(o.toObject());
+        std::copy(a.begin(), a.end(), std::back_inserter(result));
     }
 
-    auto actions = res.value(QLatin1String("potentialAction"));
-    if (!actions.isUndefined()) {
-        res.insert(QStringLiteral("potentialAction"), filterActions(actions));
+    return result;
+}
+
+QJsonArray JsonLdImportFilter::filterObject(const QJsonObject &obj)
+{
+    QStringList types;
+    const auto typeVal = obj.value(QLatin1String("@type"));
+    if (typeVal.isString()) {
+        types.push_back(typeVal.toString());
+    } else if (typeVal.isArray()) {
+        const auto typeNames = typeVal.toArray();
+        for (const auto &t : typeNames) {
+            if (t.isString()) {
+                types.push_back(t.toString());
+            }
+        }
+    }
+    // TODO consider additionalTypes property
+
+    if (types.isEmpty()) {
+        return graphExpand(obj);
+    }
+
+    QJsonArray results;
+
+    for (const auto &type : types) {
+        QJsonObject res(obj);
+        res.insert(QStringLiteral("@type"), type);
+        filterRecursive(res);
+
+        if (type.endsWith(QLatin1String("Reservation"))) {
+            filterReservation(res);
+        }
+
+        auto actions = res.value(QLatin1String("potentialAction"));
+        if (!actions.isUndefined()) {
+            res.insert(QStringLiteral("potentialAction"), filterActions(actions));
+        }
+
+        auto image = res.value(QLatin1String("image"));
+        if (image.isArray()) {
+            res.insert(QStringLiteral("image"), image.toArray().first());
+        }
+
+        image = res.value(QLatin1String("image"));
+        if (image.isObject()) {
+            const auto imageObject = image.toObject();
+            if (imageObject.value(QLatin1String("@type")).toString() == QLatin1String("ImageObject")) {
+                res.insert(QStringLiteral("image"), imageObject.value(QLatin1String("url")));
+            }
+        }
+
+        results.push_back(res);
     }
 
-    return res;
+    return results;
 }
